@@ -19,10 +19,7 @@ For more information, please see F<sepia/index.html>.
 
 =cut
 
-$VERSION = '0.90_02';
-@ISA = qw(Exporter);
-
-require Exporter;
+$VERSION = '0.94_01';
 use strict;
 use B;
 use Sepia::Debug;               # THIS TURNS ON DEBUGGING INFORMATION!
@@ -60,6 +57,45 @@ BEGIN {
     } else {
         *Sepia::core_version = sub { Module::CoreList->first_release(@_) };
     }
+    eval { use List::Util 'max' };
+    if ($@) {
+        *Sepia::max = sub {
+            my $ret = shift;
+            for (@_) {
+                $ret = $_ if $_ > $ret;
+            }
+            $ret;
+        };
+    }
+    eval { require Devel::Size };
+    if ($@) {
+        *Sepia::repl_size = sub {
+            print STDERR "Size requires Devel::Size.\n";
+            0;
+        };
+    } else {
+        *Sepia::repl_size = sub {
+            ## XXX: C&P from repl_who:
+            my ($pkg, $re) = split ' ', shift || '';
+            if ($pkg =~ /^\/(.*)\/?$/) {
+                $pkg = $PACKAGE;
+                $re = $1;
+            } elsif (!$re && !defined %{$pkg.'::'}) {
+                $re = $pkg;
+                $pkg = $PACKAGE;
+            }
+            my @who = who($pkg, $re);
+            my $len = max(map { length } @who) + 4;
+            my $fmt = '%-'.$len."s%10d\n";
+            print 'Var', ' ' x ($len + 2), "Bytes\n";
+            print '-' x ($len-4), ' ' x (9+$len), '-' x 5, "\n";
+            for (@who) {
+                my $res = eval "package $pkg; Devel::Size::total_size \\$_;";
+                printf $fmt, $_, $res || 0;
+            }
+            0;
+        };
+    }
 }
 
 =head1 DESCRIPTION
@@ -137,6 +173,10 @@ BEGIN {
     %sigil = qw(ARRAY @ SCALAR $ HASH %);
 }
 
+## XXX: autovivification gives us problems here sometimes.  Specifically:
+##     defined *FOO{HASH} # => ''
+##     defined %FOO       # => ''
+##     defined *FOO{HASH} # => 1
 sub completions
 {
     no strict;
@@ -605,7 +645,7 @@ sub printer
     if ($ISEVAL) {
         print ';;;', length $res, "\n$res\n";
     } else {
-        print "=> $res\n";
+        print "$res\n";
     }
 }
 
@@ -658,6 +698,7 @@ BEGIN {
     $PRINT_PRETTY = 1;
     %REPL = (help => \&Sepia::repl_help,
              cd => \&Sepia::repl_chdir,
+             pwd => \&Sepia::repl_pwd,
              methods => \&Sepia::repl_methods,
              package => \&Sepia::repl_package,
              who => \&Sepia::repl_who,
@@ -667,12 +708,14 @@ BEGIN {
              quit => \&Sepia::repl_quit,
              reload => \&Sepia::repl_reload,
              shell => \&Sepia::repl_shell,
-             debug => \&Sepia::Debug::repl_debug,
-             break => \&Sepia::Debug::repl_break,
+             eval => \&Sepia::repl_eval,
+             size => \&Sepia::repl_size,
          );
-    %REPL_DOC = (
+    %Sepia::REPL_DOC = (
         cd =>
     'cd DIR             Change directory to DIR',
+        pwd =>
+    'pwd                Show current working directory',
         format =>
     'format [dumper|dump|yaml|plain]
                        Set output formatter (default: dumper)',
@@ -688,6 +731,10 @@ EOS
     'quit               Quit the REPL',
         shell =>
     'shell CMD ...      Run CMD in the shell.',
+        size => <<EOS,
+size PACKAGE [RE]   List total sizes of objects in PACKAGE matching optional
+                       pattern RE.
+EOS
         strict =>
     'strict [0|1]       Turn \'use strict\' mode on or off',
         wantarray =>
@@ -699,7 +746,6 @@ EOS
         reload =>
     'reload             Reload Sepia.pm and relaunch the REPL.',
     );
-    %RK = abbrev keys %REPL;
 }
 
 sub prompt()
@@ -707,18 +753,27 @@ sub prompt()
     "$PACKAGE ".($WANTARRAY ? '@' : '$').$PS1
 }
 
-sub Dump {
+sub Dump
+{
     eval {
         Data::Dumper->Dump([$_[0]], [$_[1]]);
     };
 }
 
+sub flow
+{
+    my $n = shift;
+    local $_ = shift;
+    s/(.{$n,})? /$1\n/g;
+    $_
+}
+
 sub repl_help
 {
     print "REPL commands (prefixed with ','):\n";
     for (sort keys %REPL) {
-        print "    ",
-            exists $REPL_DOC{$_} ? "$REPL_DOC{$_}\n": "$_    (undocumented)\n";
+        print "    ", exists $REPL_DOC{$_} ? "$REPL_DOC{$_}\n":
+            sprintf("%-18s (undocumented)\n", $_);
     }
     0;
 }
@@ -744,7 +799,6 @@ sub repl_chdir
     $dir =~ s/^~\//$ENV{HOME}\//;
     $dir =~ s/\$HOME/$ENV{HOME}/;
     if (-d $dir) {
-
         chdir $dir;
         my $ecmd = '(cd "'.Cwd::getcwd().'")';
         print ";;;###".length($ecmd)."\n$ecmd\n";
@@ -754,6 +808,12 @@ sub repl_chdir
     0;
 }
 
+sub repl_pwd
+{
+    print Cwd::getcwd(), "\n";
+    0;
+}
+
 sub who
 {
     my ($pack, $re) = @_;
@@ -792,6 +852,13 @@ sub columnate
 sub repl_who
 {
     my ($pkg, $re) = split ' ', shift;
+    if ($pkg =~ /^\/(.*)\/?$/) {
+        $pkg = $PACKAGE;
+        $re = $1;
+    } elsif (!$re && !defined %{$pkg.'::'}) {
+        $re = $pkg;
+        $pkg = $PACKAGE;
+    }
     print columnate who($pkg || $PACKAGE, $re);
     0;
 }
@@ -816,7 +883,7 @@ sub repl_methods
     $x =~ s/^\s+//;
     $x =~ s/\s+$//;
     if ($x =~ /^\$/) {
-        $x = repl_eval("ref $x");
+        $x = $REPL{eval}->("ref $x");
         return 0 if $@;
     }
     $re ||= '.?';
@@ -877,9 +944,9 @@ sub repl_shell
 
 sub repl_eval
 {
-    my ($buf, $wantarray, $pkg) = @_;
+    my ($buf) = @_;
     no strict;
-    local $PACKAGE = $pkg || $PACKAGE;
+    # local $PACKAGE = $pkg || $PACKAGE;
     if ($STRICT) {
         if (!$WANTARRAY) {
             $buf = 'scalar($buf)';
@@ -888,7 +955,7 @@ sub repl_eval
         $ctx = $ctx ? "my ($ctx);" : '';
         $buf = eval "sub { package $PACKAGE; use strict; $ctx $buf }";
         if ($@) {
-            print STDERR "ERROR\n$@\n";
+            print "ERROR\n$@\n";
             return;
         }
         $STRICT->call($buf);
@@ -930,6 +997,9 @@ sub repl
     if (@_ > 0) {
         $REPL_IN = $_[0];
         $REPL_OUT = $_[1];
+    } else {
+        $REPL_IN = \*STDIN;
+        $REPL_OUT = \*STDOUT;
     }
     select $REPL_OUT;
     $| = 1;
@@ -944,11 +1014,11 @@ sub repl
 
     local *__;
     local *CORE::GLOBAL::die = \&Sepia::Debug::die;
-
     local *CORE::GLOBAL::warn = \&Sepia::Debug::warn;
+    Sepia::Debug::add_repl_commands;
     print <<EOS if $REPL_LEVEL == 1;
 Sepia version $Sepia::VERSION.
-Press ",h" for help, or "^D" or ",q" to exit.
+Type ",h" for help, or ",q" to quit.
 EOS
     print prompt;
     my @sigs = qw(INT TERM PIPE ALRM);
@@ -1005,7 +1075,7 @@ EOS
                 }
             } else {
                 ## Ordinary eval
-                @res = repl_eval $buf, wantarray;
+                @res = $REPL{eval}->($buf);
                 if ($@) {
                     if ($ISEVAL) {
                         ## Always return results for an eval request
@@ -1014,7 +1084,7 @@ EOS
                         # print_warnings $ISEVAL;
                         $buf = '';
                         print prompt;
-                    } elsif ($@ =~ /at EOF$/m) {
+                    } elsif ($@ =~ /(?:at|before) EOF(?:$| at)/m) {
                         ## Possibly-incomplete line
                         if ($in eq "\n") {
                             print "Error:\n$@\n*** cancel ***\n", prompt;
@@ -1032,7 +1102,7 @@ EOS
                     next repl;
                 }
             }
-            if ($buf !~ /;$/ && $buf !~ /^,/) {
+            if ($buf !~ /;\s*$/ && $buf !~ /^,/) {
                 ## Be quiet if it ends with a semicolon, or if we
                 ## executed a shortcut.
                 Sepia::printer \@res, wantarray;
@@ -1045,7 +1115,7 @@ EOS
 
 sub perl_eval
 {
-    tolisp(repl_eval(shift));
+    tolisp($REPL{eval}->(shift));
 }
 
 =head2 C<$status = html_module_list($file [, $prefix])>
