@@ -39,36 +39,84 @@ sub repl_backtrace
 # return value from die
 sub repl_return
 {
-    (1, Sepia::repl_eval(@_));
+    (1, $Sepia::REPL{eval}->(@_));
 }
 
-
+sub repl_lsbreak
+{
+    no strict 'refs';
+    for my $file (sort grep /^_</ && defined %{"::$_"}, keys %::) {
+        my ($name) = $file =~ /^_<(.*)/;
+        my @pts = keys %{"::$file"};
+        next unless @pts;
+        print "$name:\n";
+        for (sort { $a <=> $b } @pts) {
+            print "\t$_\t${$file}{$_}\n"
+        }
+    }
+}
 
 # evaluate EXPR in environment ENV
 sub eval_in_env
 {
     my ($expr, $env) = @_;
-    local $::SEPIA_ENV = $env;
+    local $Sepia::ENV = $env;
     my $str = '';
     for (keys %$env) {
         next unless /^([\$\@%])(.+)/;
-        $str .= "local *$2 = \$::SEPIA_ENV->{'$_'}; ";
+        $str .= "local *$2 = \$Sepia::ENV->{'$_'}; ";
     }
     eval "do { no strict; $str $expr }";
 }
 
-## XXX: this is a better approach (the local business above is vile),
+sub tie_class
+{
+    my $sig = substr shift, 0, 1;
+    return $sig eq '$' ? 'Tie::StdScalar'
+        : $sig eq '@' ? 'Tie::StdArray'
+            : $sig eq '%' ? 'Tie::StdHash'
+                : die "Sorry, can't tie $sig\n";
+}
+
+# {
+#     require Tie::Array;
+#     require Tie::Hash;
+#     require Tie::Scalar;
+#     package Sepia::Array;
+#     our @ISA = qw(Tie::StdArray);
+#     sub TIEARRAY { bless $_[1], $_[0] }
+#     package Sepia::Hash;
+#     our @ISA = qw(Tie::StdHash);
+#     sub TIEHASH { bless $_[1], $_[0] }
+#     package Sepia::Scalar;
+#     our @ISA = qw(Tie::StdScalar);
+#     sub TIESCALAR { bless $_[1], $_[0] }
+# }
+
+# sub eval_in_env3
+# {
+#     my ($expr, $env) = @_;
+#     my @vars = grep /^([\$\@%])(.+)/, keys %$env;
+#     my $body = 'sub { my ('.join(',', @vars).');';
+#     for my $i (0..$#vars) {
+#         $body .= "tie $vars[$i], ".tie_class($vars[$i]).', $_['.$i.'];';
+#     }
+#     $body .= "$expr }";
+#     print STDERR "---\n$body\n---\n";
+#     $body = eval $body;
+#     $@ || $body->(@{$env}{@vars});
+# }
+
+## XXX: this is a better approach (the local/tie business is vile),
 ## but it segfaults and I'm not sure why.
 sub eval_in_env2
 {
-    my ($expr, $lev) = @_;
-    my $env = peek_my(2+$lev);
-    $lev += 4;
-    local $::SEPIA_ENV = $env;
+    my ($expr, $env, $fn) = @_;
+    local $Sepia::ENV = $env;
     my @vars = grep /^([\$\@%])(.+)/, keys %$env;
     my $body = 'sub { my ('.join(',', @vars).');';
     for (@vars) {
-        $body .= "Devel::LexAlias::lexalias($lev, '$_', \\$_);"
+        $body .= "Devel::LexAlias::lexalias(\$Sepia::ENV, '$_', \\$_);"
     }
     $body .= "$expr }";
     print STDERR "---\n$body\n---\n";
@@ -81,15 +129,21 @@ sub repl_upeval
 {
     my $exp = shift;
     # my ($lev, $exp) = $_[0] =~ /^\s*(\d+)\s+(.*)/;
-    print " <= $exp\n";
+    # print " <= $exp\n";
     # (0, eval_in_env2($exp, $level));
-    (0, eval_in_env($exp, peek_my(0+$level)));
+    # (0, eval_in_env3($exp, peek_my(4 + $level)));
+    eval_in_env($exp, peek_my(4+$level));
 }
 
 # inspect lexicals at level N, or current level
 sub repl_inspect
 {
-    my $i = shift || $level;
+    my $i = shift;
+    if ($i =~ /\d/) {
+        $i = 0+$i;
+    } else {
+        $i = $level + 3;
+    }
     my $sub = (caller $i)[3];
     if ($sub) {
         my $h = peek_my($i+1);
@@ -117,16 +171,26 @@ sub debug
     }
 }
 
+sub breakpoint_file
+{
+    my ($file) = @_;
+    return \%{$main::{"_<$file"}} if exists $main::{"_<$file"};
+    if ($file !~ /^\//) {
+        ($file) = grep /^_<.*\/\Q$file\E$/, keys %main::;
+        return \%{$main::{$file}} if $file;
+    }
+    return undef;
+}
+
 sub breakpoint
 {
     my ($file, $line, $cond) = @_;
-    if (!defined $main::{"_<$file"} && $file !~ /^\//) {
-        ($file) = grep /^_<.*\/\Q$file\E$/, keys %main::;
-        return unless $file;
-        $file =~ s/^_<//;
+    my $h = breakpoint_file $file;
+    if (defined $h) {
+        $h->{$line} = $cond || 1;
+        return $cond ? "$file\:$line if $cond" : "$file\:$line";
     }
-    $main::{"_<$file"}{$line} = $cond || 1;
-    $file;
+    return undef;
 }
 
 sub repl_break
@@ -134,8 +198,8 @@ sub repl_break
     my $arg = shift;
     $arg =~ s/^\s+//;
     $arg =~ s/\s+$//;
-    my ($f, $l, $cond) = $arg =~ /^(.+?):(\d+)\s*(.*)/, $arg;
-    $cond ||= 1;
+    my ($f, $l, $cond) = $arg =~ /^(.+?):(\d+)\s*(.*)/;
+    $cond = 1 unless $cond =~ /\S/;
     $f ||= $file;
     $l ||= $line;
     print "break ", breakpoint($f, $l, $cond), "\n";
@@ -153,14 +217,51 @@ sub show_location
     print "_<$file:$line>\n" if defined $file && defined $line;
 }
 
-my %REPL = (
-    delete => sub {
-        my ($f, $l) = split /:/, shift;
-        $f ||= $file;
-        $l ||= $line;
-        delete $main::{"_<$f"}{$l}; 0
-    },
+sub repl_list
+{
+    my @lines = eval shift;
+    @lines = $line - 5 .. $line + 5 unless @lines;
+    printf '%-6d%s', $_, ${"::_<$file"}[$_-1] for @lines;
+    0
+}
 
+sub repl_delete
+{
+    my ($f, $l) = split /:/, shift;
+    $f ||= $file;
+    $l ||= $line;
+    my $h = breakpoint_file $f;
+    delete $h->{$l} if defined $h;
+    0
+}
+
+my %parent_repl = (
+    delete => \&repl_delete,
+    debug => \&repl_debug,
+    break => \&repl_break,
+    lsbreak => \&repl_lsbreak,
+);
+
+my %parent_doc = (
+    break =>
+        'break [F:N [E]]    Set a breakpoint in F at line N (or at current
+                       position), enabled if E evalutes to true.',
+    delete =>
+        'delete             Delete current breakpoint.',
+    debug =>
+        'debug [0|1]        Enable or disable debugging.',
+    lsbreak =>
+        'lsbreak            List breakpoints.',
+);
+
+sub add_repl_commands
+{
+    %Sepia::REPL = (%Sepia::REPL, %parent_repl);
+    %Sepia::REPL_DOC = (%Sepia::REPL_DOC, %parent_doc);
+    %Sepia::RK = abbrev keys %Sepia::REPL;
+}
+
+my %REPL = (
     up => sub {
         $level += shift || 1;
         update_location(4);
@@ -192,29 +293,22 @@ my %REPL = (
 
     break => \&repl_break,
 
-    list => sub {
-        my @lines = eval shift;
-        print join('', @{$main::{"_<$file"}}[@lines]);
-        0
-    },
+    list => \&repl_list,
 
     # quit => sub {
     #     debug(0);
     # },
     backtrace => \&repl_backtrace,
     inspect => \&repl_inspect,
-    eval => \&repl_upeval,
+    # eval => \&repl_upeval,
     return => \&repl_return,
+    lsbreak => \&repl_lsbreak,
+    eval => \&repl_upeval,      # DANGER!
 );
 
 my %REPL_DOC = (
-    break =>
-        'break [F:N [E]] Set a breakpoint in F at line N (or at current
-                       position), enabled if E evalutes to true.',
     continue =>
         'continue        Yep.',
-    delete =>
-        'delete          Delete current breakpoint.',
     next =>
         'next [N]        Advance N lines, skipping subroutines.',
     list =>
@@ -242,8 +336,6 @@ my %REPL_DOC = (
 sub repl
 {
     show_location;
-    # print STDERR $main::{'_<'.$file}[$line]
-    #     if defined $main::{'_<'.$file}[$line];
 
     local %Sepia::REPL = (%Sepia::REPL, %REPL, @_);
     local %Sepia::REPL_DOC = (%Sepia::REPL_DOC, %REPL_DOC);
@@ -287,8 +379,8 @@ sub die
         my $trace = $DB::trace;
         $DB::trace = 1;
         repl(
-            die => sub { local $STOPDIE=0; die @dieargs },
-            quit => sub { local $STOPDIE=0; die @dieargs });
+            die => sub { local $STOPDIE=0; CORE::die @dieargs },
+            quit => sub { local $STOPDIE=0; CORE::die @dieargs });
         $DB::trace = $trace;
     } else {
         CORE::die(Carp::shortmess @_);
@@ -306,8 +398,8 @@ sub warn
         local ($pack, $file, $line, $sub) = caller($level);
         print "@_\n\tin $sub\nWarned $MSG\n";
         repl(
-            warn => sub { local $STOPWARN=0; warn @dieargs },
-            quit => sub { local $STOPWARN=0; warn @dieargs });
+            warn => sub { local $STOPWARN=0; CORE::warn @dieargs },
+            quit => sub { local $STOPWARN=0; CORE::warn @dieargs });
         $DB::trace = $trace;
     } else {
         ## Avoid showing up in location information.
